

#define TRUE		1
#define FALSE		0

#include <float.h>
#define PLUS_INFINITY	DBL_MAX
#define MINUS_INFINITY	DBL_MIN

/* minimization of M(#)-convex function   */
/* returns the minimum value of func(x)   */
/* INPUT  func  : pointer to the function */
/* INPUT  init[] : the initial solution   */
/* OUTPUT init[] : the minimized solution */
/* steepest descent algorithm */
double  mconv_minimize(int dim, double func(int dim, int x[]),
                       int init[], int lower[], int upper[]);
double mgconv_minimize(int dim, double func(int dim, int x[]),
                       int init[], int lower[], int upper[]);
/* greedy algorithm (improved steepest descent method) */
double  mconv_minimize2(int dim, double func(int dim, int x[]), int init[]);
double mgconv_minimize2(int dim, double func(int dim, int x[]), int init[]);
/* greedy algorithm (improved steepest descent method) */
double  mconv_minimize3(int dim, double func(int dim, int x[]),
                        int init[], int lower[], int upper[]);
double mgconv_minimize3(int dim, double func(int dim, int x[]),
                        int init[], int lower[], int upper[]);
/* scaling algorithm (with greedy) */
double  mconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[], int lower[], int upper[]);
double mgconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[], int lower[], int upper[]);
/* continues relaxation algorithm */
double  mconv_minimize_relax(int dim, double func(int dim, double x[]),
                             int init[]);
double mgconv_minimize_relax(int dim, double func(int dim, double x[]),
			     int init[]);

/* minimization of L(#)-convex function   */
/* returns the minimum value of func(x)   */
/* INPUT  func  : pointer to the function */
/* INPUT  init[] : the initial solution   */
/* OUTPUT init[] : the minimized solution */

/* steepest descent algorithm (experimental) */
double  lconv_minimize_type(int dim, double func(int dim, int x[]), int init[],
			    const int fix_index, const int type);
double lgconv_minimize_type(int dim, double func(int dim, int x[]), int init[],
			    const int type);

/* steepest descent algorithm */
double  lconv_minimize(int dim, double func(int dim, int x[]), int init[],
		       const int fix_index);
double lgconv_minimize(int dim, double func(int dim, int x[]), int init[]);
/* steepest descent scaling algorithm */
double  lconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[], const int fix_index);
double lgconv_minimize_scaling(int dim, double func(int dim, int x[]),
                               int init[]);
/* steepest descent algorithm (with IFF) */
double  lconv_minimize_IFF(int dim, double func(int,int*), int init[],
                           const int fix_index);
double lgconv_minimize_IFF(int dim, double func(int,int*), int init[]);
/* steepest descent scaling algorithm (with IFF) */
double  lconv_minimize_scaling_IFF(int dim, double func(int,int*), int init[],
                           const int fix_index);
double lgconv_minimize_scaling_IFF(int dim, double func(int,int*), int init[]);
/* steepest descent algorithm (with FW) */
double  lconv_minimize_FW(int dim, double func(int,int*), int init[],
                           const int fix_index);
double lgconv_minimize_FW(int dim, double func(int,int*), int init[]);
/* steepest descent scaling algorithm (with FW) */
double  lconv_minimize_scaling_FW(int dim, double func(int,int*),int init[],
                           const int fix_index);
double lgconv_minimize_scaling_FW(int dim, double func(int,int*),int init[]);
/* continues relaxation algorithm (with IFF) */
double  lconv_minimize_relax(int dim, double func(int dim, double x[]),
                             int init[], const int fix_index);
double lgconv_minimize_relax(int dim, double func(int dim, double x[]),
			     int init[]);

/* decide if a function is L#/M# convex (by Hessian on all points) */
int is_lgconv_gene(int dim, double func(int,int*), int lower[], int upper[]);
int  is_lconv_gene(int dim, double func(int,int*), int lower[], int upper[]);
int is_mgconv_gene(int dim, double func(int,int*), int lower[], int upper[]);
int  is_mconv_gene(int dim, double func(int,int*), int lower[], int upper[],
                   int sum);

extern enum verbose_e {
    LOOP=1,  STATUS=2,  COUNTER=4,  MINIMIZER=8,
} verbose_level;
extern int oracle_call, improve_call, iteration;
extern int    with_limit;
extern double limit_cpu_time;
extern int    limit_iteration;
extern double limit_abs_func;

void verbose_level_set(enum verbose_e level);
enum verbose_e verbose_level_get(void);
void verbose_level_on (enum verbose_e level);
void verbose_level_off(enum verbose_e level);

void counter_reset(void);
void counter_print(void);
void counter_println(void);
void minimizer_print(int dim, double func(int,int*), int x[]);

extern enum submodular_e { IFF, FW, IFF_FW } submodular_type;

int   is_feasible(int dim, double func(int,int*), int x[]);
void   loop_print(int dim, double func(int,int*), int x[]);
void status_print(int dim, double func(int,int*), int x[]);

int fcmp(double a, double b);
int iround(double x);
double int_func(int dim, double double_func(int,double*), int x[]);
double intint_func(int dim, double double_func(int,double*), int x[]);

void *xmalloc(size_t size);

/* constractor of vector */
int    *ivector_new(const int dim, ...);
double *fvector_new(const int dim, ...);

/* print vector */
void ivector_print(const int dim, const int    x[]);
void fvector_print(const int dim, const double x[]);

/* copy vector */
int    *ivector_copy(const int dim, int    *dst, const int    *src);
double *fvector_copy(const int dim, double *dst, const double *src);

/* compare vector */
int ivector_comp(const int dim, int    *dst, const int    *src);
int fvector_comp(const int dim, double *dst, const double *src);

/* constractor of matrix */
int    **imatrix_new(int m, int n);
double **fmatrix_new(int m, int n);

int ivector_scan(const int dim, int    *a);
int fvector_scan(const int dim, double *a);
int imatrix_scan(const int dim, int    **aa);
int fmatrix_scan(const int dim, double **aa);

void fmatrix_print(const int dim, double **aa);

/* [min,max] random */
void mt_initialize(long seed);
int    irandom(int    min, int    max);
double frandom(double min, double max);

/* math func */
double factorial(int n);

/* CPU time */
double cpu_time(void);

/* L# convex function minimization as submodular function */
double sf_minimize(int dim, double func(int,int*), int x[], int direction,
                   int set[], const int fix_index);

double lbfgs_call(int dim, double func(int dim, double x[]), double init[]);
double lbfgs_call_int(int dim, double func(int dim, double x[]), int init[]);

/* loop for high dimension, typically domain */
int next(int dim, int x[], int lower[], int upper[], int step);

/* unit test routine */
void test_mconv(const int dim);
void test_mconv2(void);
void test_lconv(const int dim);
void test_lconv2(const int dim, int argc, char** argv);
void test_lconv3(const int dim, int argc, char** argv);
void test_qlconv(const int dim);
void test_lconv_dist(const int dim);
void test_submodular(const int dim);
void test_quadratic(const int dim);
void test_convexity(const int dim);

void demo_Lseparable(const int dim, char type);
void demo_Mseparable(const int dim, char type);
void demo_quadratic(const int dim, char MorL);
void demo_inventory(const int dim);
void demo_inventory2(const int dim);
void demo_shift_schedule(const int dim);
void demo_shift_schedule2(const int dim);
